@@ -31,8 +31,7 @@
 #include "ice.h"
 #include "logout.h"
 #include "command.h"
-
-#define SESSION_STOCK_LOGOUT "session-logout"
+#include "util.h"
 
 static gchar *halt_command[] =
 {
@@ -69,31 +68,15 @@ iris_callback (gpointer data)
   height_step = MIN (iris_rect.width / 2, iris_block);
 
   for (i = 0; i < MIN (width_step, height_step); i++)
-    {
-      GdkPoint points[5];
-      static gchar dash_list[2] =
-      {1, 1};
-
-      gdk_gc_set_dashes (iris_gc, i % 1, dash_list, 2);
-
-      points[0].x = iris_rect.x + i;
-      points[0].y = iris_rect.y + i;
-      points[1].x = iris_rect.x + iris_rect.width - i;
-      points[1].y = iris_rect.y + i;
-      points[2].x = iris_rect.x + iris_rect.width - i;
-      points[2].y = iris_rect.y + iris_rect.height - i;
-      points[3].x = iris_rect.x + i;
-      points[3].y = iris_rect.y + iris_rect.height - i;
-      points[4].x = iris_rect.x + i;
-      points[4].y = iris_rect.y + i;
-
-      gdk_draw_lines (GDK_ROOT_PARENT (), iris_gc, points, 5);
-    }
+    gdk_draw_rectangle (GDK_ROOT_PARENT (), iris_gc, FALSE,
+			iris_rect.x + i, iris_rect.y + i,
+			(gint)iris_rect.width - 2 * i,
+			(gint)iris_rect.height - 2 * i);
 
   gdk_flush ();
 
   iris_rect.x += width_step;
-  iris_rect.y += width_step;
+  iris_rect.y += height_step;
   iris_rect.width -= MIN (iris_rect.width, iris_block * 2);
   iris_rect.height -= MIN (iris_rect.height, iris_block * 2);
 
@@ -117,6 +100,7 @@ iris (void)
   if (!iris_gc)
     {
       GdkGCValues values;
+      static gchar dash_list[2] = {1, 1};
 
       values.line_style = GDK_LINE_ON_OFF_DASH;
       values.subwindow_mode = GDK_INCLUDE_INFERIORS;
@@ -124,6 +108,8 @@ iris (void)
       iris_gc = gdk_gc_new_with_values (GDK_ROOT_PARENT (),
 					&values,
 				      GDK_GC_LINE_STYLE | GDK_GC_SUBWINDOW);
+
+      gdk_gc_set_dashes (iris_gc, 0, dash_list, 2);
     }
 
   /* Plan for a time of 0.5 seconds for effect */
@@ -152,7 +138,7 @@ refresh_screen (void)
   attributes.wclass = GDK_INPUT_OUTPUT;
   attributes.override_redirect = TRUE;
   attributes.event_mask = 0;
-  
+
   window = gdk_window_new (NULL, &attributes,
 			   GDK_WA_X | GDK_WA_Y | GDK_WA_NOREDIR);
 
@@ -187,29 +173,6 @@ force_pango_cache_init ()
 	pango_font_description_free (font_desc);
 }
 
-static inline void
-register_logout_stock_item (void)
-{
-	static gboolean registered = FALSE;
-
-	if (!registered) {
-		GtkIconFactory      *factory;
-		GtkIconSet          *quit_icons;
-
-		static GtkStockItem  logout_item [] = {
-			{ SESSION_STOCK_LOGOUT, N_("_Log Out"), 0, 0, GETTEXT_PACKAGE },
-		};
-
-		quit_icons = gtk_icon_factory_lookup_default (GTK_STOCK_QUIT);
-		factory = gtk_icon_factory_new ();
-		gtk_icon_factory_add (factory, SESSION_STOCK_LOGOUT, quit_icons);
-		gtk_icon_factory_add_default (factory);
-		gtk_stock_add_static (logout_item, 1);
-
-		registered = TRUE;
-	}
-}
-
 static gboolean
 display_gui (void)
 {
@@ -221,6 +184,11 @@ display_gui (void)
   GtkWidget *reboot = NULL;
   GtkWidget *invisible;
   gboolean   retval = FALSE;
+  gboolean save_active = FALSE;
+  gboolean halt_active = FALSE;
+  gboolean reboot_active = FALSE;
+
+  gsm_verbose ("display_gui: showing logout dialog\n");
 
   /* It's really bad here if someone else has the pointer
    * grabbed, so we first grab the pointer and keyboard
@@ -250,8 +218,6 @@ display_gui (void)
   XGrabServer (GDK_DISPLAY ());
   iris ();
 
-  register_logout_stock_item ();
-
   box = gtk_message_dialog_new (NULL, 0,
 				GTK_MESSAGE_QUESTION,
 				GTK_BUTTONS_NONE,
@@ -259,7 +225,7 @@ display_gui (void)
 
   gtk_dialog_add_button (GTK_DIALOG (box), GTK_STOCK_HELP, GTK_RESPONSE_HELP);
   gtk_dialog_add_button (GTK_DIALOG (box), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-  gtk_dialog_add_button (GTK_DIALOG (box), SESSION_STOCK_LOGOUT, GTK_RESPONSE_OK);
+  gtk_dialog_add_button (GTK_DIALOG (box), GTK_STOCK_OK, GTK_RESPONSE_OK);
 
   g_object_set (G_OBJECT (box), "type", GTK_WINDOW_POPUP, NULL);
 
@@ -321,6 +287,17 @@ display_gui (void)
 
   response = gtk_dialog_run (GTK_DIALOG (box));
 
+  if (halt)
+    halt_active = GTK_TOGGLE_BUTTON (halt)->active;
+
+  if (reboot)
+    reboot_active = GTK_TOGGLE_BUTTON (reboot)->active;
+
+  if (toggle_button)
+    save_active = GTK_TOGGLE_BUTTON (toggle_button)->active;
+
+  gtk_widget_destroy (box);
+
   refresh_screen ();
   XUngrabServer (GDK_DISPLAY ());
 
@@ -333,16 +310,12 @@ display_gui (void)
     case GTK_RESPONSE_OK:
       /* We want to know if we should trash changes (and lose forever)
        * or save them */
-      if(!autosave)
-        save_selected = GTK_TOGGLE_BUTTON (toggle_button)->active;
-
-      if (halt)
-	{
-	  if (GTK_TOGGLE_BUTTON (halt)->active)
-	    action = HALT;
-	  else if (GTK_TOGGLE_BUTTON (reboot)->active)
-	    action = REBOOT;
-	}
+      if(save_active)
+	save_selected = save_active;
+      if (halt_active)
+	action = HALT;
+      else if (reboot_active)
+	action = REBOOT;
       retval = TRUE;
       break;
     default:
@@ -355,8 +328,6 @@ display_gui (void)
       break;
     }
 
-  gtk_widget_destroy (box);
-
   return retval;
 }
 
@@ -365,8 +336,11 @@ display_gui (void)
 gboolean
 maybe_display_gui (void)
 {
-  gboolean result, prompt = gnome_config_get_bool (GSM_OPTION_CONFIG_PREFIX "LogoutPrompt=true");
-  if (! prompt)
+  gboolean result;
+
+  gsm_verbose ("maybe_display_gui(): showing dialog %d\n", logout_prompt);
+
+  if (!logout_prompt)
     return TRUE;
 
   /*
