@@ -39,6 +39,7 @@ extern int errno;
 #include "logout.h"
 #include "ice.h"
 #include "gsm-protocol.h"
+#include "util.h"
 
 /* The save_state gives a description of what we are doing */
 static enum {
@@ -128,6 +129,9 @@ gchar   *events[]       = { GsmInactive, GsmInactive,
 
 static gint compare_interact_request (gconstpointer a, gconstpointer b);
 
+/* whether our current save is global */
+static gboolean global_save;
+
 typedef void message_func (SmsConn connection);
 
 static void
@@ -145,7 +149,36 @@ set_properties (SmsConn connection, SmPointer data,
 
 static void update_save_state (void);
 
+static void
+maybe_write_session (void)
+{
+  if (autosave || save_selected || session_save)
+    write_session ();
 
+  save_selected = session_save = FALSE;
+
+  if (!shutting_down && global_save)
+    {
+      static GtkWidget *dialog = NULL;
+
+      if (dialog)
+	{
+	  gtk_window_present (GTK_WINDOW (dialog));
+	}
+      else
+	{
+	  dialog = gtk_message_dialog_new (NULL, 0, 
+					   GTK_MESSAGE_INFO,
+					   GTK_BUTTONS_OK, 
+					   _("Your session has been saved"));
+	  g_signal_connect (dialog, "response",
+			    G_CALLBACK (gtk_widget_destroy),
+			    NULL);
+	  g_object_add_weak_pointer (G_OBJECT (dialog), (gpointer *)&dialog);
+	  gtk_widget_show (dialog);
+	}
+    }
+}
 
 Client *
 find_client_by_id (const GSList *list, const char *id)
@@ -231,6 +264,9 @@ void
 free_client (Client *client)
 {
  GSList *list; 
+
+ gsm_verbose ("free_client(): %p (%s)\n", client, client->id ? client->id : "(null)");
+
  if (! client)
     return;
   if(client->magic != CLIENT_MAGIC)
@@ -279,17 +315,22 @@ free_client (Client *client)
   g_slist_free (client->get_prop_replies);
   client->get_prop_replies = NULL; /* sanity */
 
+  if (client->last_discard)
+    g_free (client->last_discard);
+
   g_free (client);
 }
 
 /* Run a command on a client. */
 gint
-run_command (Client* client, const gchar* command)
+run_command_prop (Client     *client, 
+		  const char *command, 
+		  SmProp     *prop)
 {
   int argc, envc, envpc, pid = -1;
   char **argv, *dir, **envv, **envp, *restart_info;
 
-  if (find_vector_property (client, command, &argc, &argv))
+  if (gsm_parse_vector_prop (prop, &argc, &argv))
     {
       if (!find_string_property (client, SmCurrentDirectory, &dir))
 	dir = NULL;
@@ -351,6 +392,19 @@ run_command (Client* client, const gchar* command)
   return pid;
 }
 
+int
+run_command (Client      *client,
+	     const gchar *command)
+{
+  SmProp *prop;
+
+  prop = find_property_by_name (client, command);
+  if (!prop)
+    return -1;
+
+  return run_command_prop (client, command, prop);
+}
+
 /* Send a message to every client on LIST.  */
 static void
 send_message (GSList **list, message_func *message)
@@ -467,19 +521,23 @@ purge (gpointer data)
 }
 
 static void
-reincarnate_client (Client *old_client, Client *new_client)
+reincarnate_client (Client *old_client,
+		    Client *new_client)
 {
-  new_client->properties        = old_client->properties;
-  new_client->priority          = old_client->priority;
-  new_client->match_rule        = old_client->match_rule;
-  new_client->attempts          = old_client->attempts;
-  new_client->connect_time      = old_client->connect_time;
-  new_client->command_data      = old_client->command_data;
+  new_client->properties    = old_client->properties;
+  new_client->priority      = old_client->priority;
+  new_client->match_rule    = old_client->match_rule;
+  new_client->attempts      = old_client->attempts;
+  new_client->connect_time  = old_client->connect_time;
+  new_client->command_data  = old_client->command_data;
+  new_client->session_saved = old_client->session_saved;
   
   old_client->properties = NULL;
+
   if (old_client->handle)
     new_client->handle = command_handle_reassign (old_client->handle, new_client);
   old_client->handle = NULL;
+
   free_client (old_client);	  
 }
 
@@ -662,6 +720,8 @@ register_client (SmsConn connection, SmPointer data, char *previous_id)
 {
   Client *client = (Client *) data;
 
+  gsm_verbose ("register_client: %p\n", connection);
+
   if (previous_id)
     {
       Client *offspring = find_client_by_id (pending_list, previous_id);
@@ -767,6 +827,8 @@ interact_request (SmsConn connection, SmPointer data, int dialog_type)
 {
   Client *client = (Client *) data;
 
+  gsm_verbose ("interact_request: %p\n", connection);
+
   /* These seem to be the only circumstances in which interactions
      can be allowed without breaking the protocol.
      FIXME: Should we inform the user of bogus requests ? */
@@ -897,6 +959,8 @@ interact_done (SmsConn connection, SmPointer data, gboolean cancel)
 {
   Client *client = (Client *) data;
 
+  gsm_verbose ("interact_done: %p\n", connection);
+
   /* if client != interact_list->data then libSM will NOT call
      this function when the client at the head of the interact_list
      sends its SmInteractDone. Therefore, we need to remove the
@@ -938,7 +1002,14 @@ static void
 process_save_request (Client* client, int save_type, gboolean shutdown, 
 		      int interact_style, gboolean fast, gboolean global)
 {
+  gsm_verbose ("process_save_request(): client %p\n", client);
+
+  gsm_verbose ("\ttype: %d\tshutting down: %d\tinteract: %d\n"
+	       "\tfast: %d\tglobal: %d\n",
+	       save_type, shutdown, interact_style, fast, global);
+
   save_state = SENDING_MESSAGES;
+  global_save = global;
   if (global)
     {
       shutting_down = shutdown;
@@ -951,20 +1022,26 @@ process_save_request (Client* client, int save_type, gboolean shutdown,
 	  shutting_down = FALSE;
 	  if (client && client->magic != CLIENT_MAGIC)
 	    g_warning("Oh my god the dead clients are walking again, please report this!");
-	  else if (client)
-	    SmsShutdownCancelled (client->connection);
 	}
       else
 	{ 
 	  while (live_list) 
 	    {
 	      Client *tmp_client = (Client *) live_list->data;
+
 	      client_event (tmp_client->handle, GsmSave);
 	      REMOVE (live_list, tmp_client);
 	      APPEND (save_yourself_list, tmp_client);
+
+	      tmp_client->last_discard = gsm_prop_copy (
+			find_property_by_name (tmp_client, SmDiscardCommand));
+
 	      SmsSaveYourself (tmp_client->connection, save_type, shutting_down,
 			       interact_style, fast);
 	    }
+
+	  if (!shutdown)
+	    save_selected = TRUE;
 	}
     }
   else
@@ -972,6 +1049,10 @@ process_save_request (Client* client, int save_type, gboolean shutdown,
       client_event (client->handle, GsmSave);
       REMOVE (live_list, client);
       APPEND (save_yourself_list, client);
+
+      client->last_discard = gsm_prop_copy (
+		find_property_by_name (client, SmDiscardCommand));
+
       SmsSaveYourself (client->connection, 
 		       save_type, 0, interact_style, fast);
     }
@@ -996,6 +1077,9 @@ save_yourself_request (SmsConn connection, SmPointer data, int save_type,
 		       gboolean global)
 {
   Client *client = (Client *) data;
+
+  gsm_verbose ("save_yourself_request: %p\n", connection);
+  
     if (save_state != MANAGER_IDLE)
       {
         SaveRequest *request = g_new (SaveRequest, 1);
@@ -1018,6 +1102,8 @@ save_yourself_p2_request (SmsConn connection, SmPointer data)
 {
   Client *client = (Client *) data;
 
+  gsm_verbose ("save_yourself_p2_request: %p\n", connection);
+
   if (g_slist_find (save_yourself_list, client) != NULL) 
     {
       REMOVE (save_yourself_list, client);
@@ -1108,7 +1194,7 @@ update_save_state (void)
       if (shutting_down)
 	{
 	  Client *warner = get_warner();
-	  GSList* list;
+	  GSList *list;
 
 	  while (purge_drop_list)
 	    {
@@ -1118,12 +1204,13 @@ update_save_state (void)
 	      REMOVE (purge_drop_list, client);
 	      free_client (client);
 	    }
-          if(autosave || save_selected || session_save)
-	    write_session ();
+
+	  maybe_write_session ();
 
 	  for (list = save_finished_list; list;)
 	    {
-	      Client *client = (Client *)list->data;
+	      Client *client = list->data;
+
 	      list = list->next;
 	      if (client != warner)
 		SmsDie (client->connection);
@@ -1142,8 +1229,7 @@ update_save_state (void)
 	}
       else
 	{
-          if(autosave || save_selected || session_save)
-	    write_session ();
+	  maybe_write_session ();
 
 	  save_state = SENDING_MESSAGES;
 	  send_message (&save_finished_list, SmsSaveComplete);
@@ -1176,8 +1262,7 @@ update_save_state (void)
 
       CONCAT (live_list, save_finished_list);
       save_finished_list = NULL;
-      if(autosave || save_selected || session_save)
-        write_session ();
+      maybe_write_session ();
 
       save_state = MANAGER_IDLE;
     }
@@ -1208,12 +1293,41 @@ update_save_state (void)
     }
 }
 
+/*
+ * The basic rule here is that every time we
+ * 'forget' a RestartCommand we need to run
+ * the DiscardCommand.
+ */
+static void
+maybe_run_discard_commands (Client *client)
+{
+  SmProp *discard;
+
+  discard = find_property_by_name (client, SmDiscardCommand);
+
+  /* Broken Client: not saving its session data on a
+   * per-SaveYourself basis.
+   * See 'Advice to Implementors' under the SaveYourself
+   * section in the XSMP spec.
+   */
+   if (gsm_prop_compare (client->last_discard, discard))
+     return;
+
+   if (client->last_discard && !client->session_saved)
+     run_command_prop (client, SmDiscardCommand, client->last_discard);
+
+   if (discard && shutting_down && !save_selected)
+     run_command_prop (client, SmDiscardCommand, discard);
+}
+
 static void
 save_yourself_done (SmsConn connection, SmPointer data, gboolean success)
 {
   Client *client = (Client *) data;
   GSList *prop_list = NULL;
 
+  gsm_verbose ("save_yourself_done: %p\n", connection);
+
   if (save_state == MANAGER_IDLE || save_state == STARTING_SESSION ||
       (g_slist_find (save_yourself_list, client) == NULL &&
        g_slist_find (save_yourself_p2_list, client) == NULL))
@@ -1295,6 +1409,8 @@ save_yourself_done (SmsConn connection, SmPointer data, gboolean success)
       set_properties(client->connection, (SmPointer)client, nprops, props);
     }
 
+  maybe_run_discard_commands (client);
+
   REMOVE (save_yourself_list, client);
   REMOVE (save_yourself_p2_list, client);
   APPEND (save_finished_list, client);      
@@ -1312,6 +1428,8 @@ close_connection (SmsConn connection, SmPointer data, int count,
   Client *client = (Client *) data;
   IceConn ice_conn = SmsGetIceConnection (connection);
 
+  gsm_verbose ("close_connection: connection %p\n", connection);
+
   client_reasons (client, FALSE, count, reasons);
   SmFreeReasons (count, reasons);
 
@@ -1328,6 +1446,8 @@ client_clean_up (Client* client)
   int style;
   GSList *list;
   
+  gsm_verbose ("client_clean_up: client %p\n", client);
+
   command_clean_up (client);
   
   SmsCleanUp (client->connection);
@@ -1419,6 +1539,8 @@ set_properties (SmsConn connection, SmPointer data, int nprops, SmProp **props)
   Client *client = (Client *) data;
   int i;
 
+  gsm_verbose ("set_properties: %p\n", connection);
+
   if (!nprops)
     return;
 
@@ -1454,6 +1576,8 @@ delete_properties (SmsConn connection, SmPointer data, int nprops,
   Client *client = (Client *) data;
   int i;
 
+  gsm_verbose ("delete_properties: %p\n", connection);
+
   for (i = 0; i < nprops; ++i)
     {
       SmProp *prop;
@@ -1483,6 +1607,8 @@ get_properties (SmsConn connection, SmPointer data)
   Client *client = (Client *) data;
   GSList *prop_list, *list;
 
+  gsm_verbose ("get_properties: %p\n", connection);
+
   client->get_prop_requests++;
 
   if (! command_active (client))
@@ -1536,6 +1662,8 @@ new_client (SmsConn connection, SmPointer data, unsigned long *maskp,
 {
   Client *client;
 
+  gsm_verbose ("new_client(): connection %p\n", connection);
+
   if (shutting_down)
     {
       *reasons  = strdup (_("A session shutdown is in progress."));
@@ -1616,8 +1744,13 @@ save_session (int save_type, gboolean shutdown, int interact_style,
       request->global = TRUE;
       
       APPEND (save_request_list, request);
+
+      gsm_verbose ("save_session(): delaying because we are not idle\n");
     }
   else
-    process_save_request (NULL, save_type, shutdown, 
-			  interact_style, fast, TRUE);
+    {
+      gsm_verbose ("save_session(): initiating session save\n");
+      process_save_request (NULL, save_type, shutdown, 
+			    interact_style, fast, TRUE);
+    }
 }
