@@ -43,6 +43,7 @@ Atom wmSaveYourselfAtom;
 Atom wmStateAtom;
 Atom smClientIdAtom;
 Atom wmClientLeaderAtom;
+Atom dtSaveModeAtom;
 
 Bool debug = 0;
 
@@ -1428,6 +1429,16 @@ char **argv;
     wmStateAtom = XInternAtom (disp, "WM_STATE", False);
     smClientIdAtom = XInternAtom (disp, "SM_CLIENT_ID", False);
     wmClientLeaderAtom = XInternAtom (disp, "WM_CLIENT_LEADER", False);
+    wmClientLeaderAtom = XInternAtom (disp, "WM_CLIENT_LEADER", False);
+    dtSaveModeAtom = XInternAtom (disp, "_DT_SAVE_MODE", False);
+
+    /* Workaround for utterly broken CDE applications.
+     * See http://bugzilla.gnome.org/show_bug.cgi?id=81343
+     * for details of the brokeness.
+     */
+    XChangeProperty (disp, RootWindow (disp, 0),
+                     dtSaveModeAtom, XA_STRING, 8, PropModeReplace,
+		     "gnome", strlen ("gnome") + 1);
 
     for (i = 0; i < ScreenCount (disp); i++)
     {
